@@ -11,6 +11,7 @@ from prusek_spheroid import metrics as metric
 from prusek_spheroid.methods import BaseImageProcessing
 from prusek_spheroid import characteristic_functions as cf
 from prusek_spheroid import image_processing as ip
+import joblib
 
 
 class Contours(BaseImageProcessing):
@@ -19,14 +20,16 @@ class Contours(BaseImageProcessing):
                  contours_state, detect_corrupted, create_json, calculate_properties,
                  progress_window=None):
         super().__init__()
+        self.counter = None
         self.master = master
         self.user_decision_lock = Lock()
         self.adresaDatasetu = adresaDatasetu
-        self.output_json_path = f"{adresa_output}/{projekt}/CVAT/{algorithm}/annotations/instances_default.json"
-        self.output_images_path = f"{adresa_output}/{projekt}/CVAT/{algorithm}/images"
-        self.output_segmented_path = f"{adresa_output}/{projekt}/segmented_images/{algorithm}"
-        self.zipfile_address = f"{adresa_output}/{projekt}/CVAT/{algorithm}"
-        self.excel_address = f"{adresa_output}/{projekt}"
+        self.output_json_path = os.path.join(adresa_output, projekt, "CVAT", algorithm, "annotations",
+                                             "instances_default.json")
+        self.output_images_path = os.path.join(adresa_output, projekt, "CVAT", algorithm, "images")
+        self.output_segmented_path = os.path.join(adresa_output, projekt, "segmented_images", algorithm)
+        self.zipfile_address = os.path.join(adresa_output, projekt, "CVAT", algorithm)
+        self.excel_address = os.path.join(adresa_output, projekt)
         self.coco_data = fm.initialize_coco_data()
         self.show_img = show_img
         self.projekt = projekt
@@ -40,19 +43,43 @@ class Contours(BaseImageProcessing):
         self.progress_window = progress_window
         self.min_area = self.parameters["min_area"]
 
+        current_dir = os.path.dirname(__file__)  # Gets the directory where the script is located
+        try:
+            model_path = os.path.join(current_dir, 'gradient_boosting_classifier.joblib')
+            scaler_path = os.path.join(current_dir, 'scaler.joblib')
+            # Load the model and scaler
+            self.model = joblib.load(model_path)
+            self.scaler = joblib.load(scaler_path)
+        except Exception as e:
+            print(f"Loading from primary location failed: {e}")
+            print("Attempting to load from alternate location...")
+            # Attempt to load the model and scaler from the alternate location
+            # Load the model and scaler
+            model_path = os.path.join(current_dir, 'prusek_spheroid', 'gradient_boosting_classifier.joblib')
+            scaler_path = os.path.join(current_dir, 'prusek_spheroid', 'scaler.joblib')
+            self.model = joblib.load(model_path)
+            self.scaler = joblib.load(scaler_path)
+
         fm.create_directory(os.path.dirname(self.output_json_path), delete=True)
         fm.create_directory(self.output_images_path, delete=True)
-        fm.create_directory(f"{self.output_segmented_path}/masks", delete=True)
-        fm.create_directory(f"{self.output_segmented_path}/results", delete=True)
+        fm.create_directory(os.path.join(self.output_segmented_path, "masks"), delete=True)
+        fm.create_directory(os.path.join(self.output_segmented_path, "results"), delete=True)
+
+    def evaluate(self, contour):
+        hu_moments = ip.compute_hu_moments(contour)
+        hu_moments_scaled = self.scaler.transform([hu_moments])
+        contour_class = self.model.predict(hu_moments_scaled)[0]
+        return contour_class
 
     def run(self):
         dialog = None
-        all_contour_data = []
-        filenames = os.listdir(self.adresaDatasetu)
+        self.counter = 1
+        filenames = [f for f in os.listdir(self.adresaDatasetu) if
+                     f.lower().endswith(('.png', '.jpg', '.jpeg', '.bmp', '.tiff', '.tif'))]
         total_files = len(filenames)
         print(f"loaded {total_files} dataset images")
-        self.counter = 1
 
+        all_contour_data = []
         if self.contours_state == "select":
             dialog = sd.SelectionDialog(self.master, self.counter, total_files, self.user_decision_lock)
 
@@ -69,29 +96,30 @@ class Contours(BaseImageProcessing):
                     continue  # Skip to the next image
 
                 img_gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)
+                img_gray = cv.resize(img_gray, (1000, 1000), interpolation=cv.INTER_LANCZOS4)
 
                 img_binary, inner_contours_mask = self.apply_segmentation_algorithm(self.algorithm, self.parameters,
                                                                                     img_gray,
-                                                                                    self.contours_state,
-                                                                                    self.detect_corrupted)
-
+                                                                                    self.contours_state)
 
+                outer_contours, _ = cv.findContours(img_binary, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
+                inner_contours = []
                 if self.contours_state == "all" or self.contours_state == "select":
-                    intersection = inner_contours_mask & img_binary
-                    img_binary = img_binary - intersection
-
-                contours, hierarchy = cv.findContours(img_binary, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
+                    inner_contours, _ = cv.findContours(inner_contours_mask, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
+                    inner_contours = ip.remove_small_contours(inner_contours)
 
                 height, width = np.shape(img_binary)
 
-                contours, hierarchy = ip.filter_contours_on_frame(contours, hierarchy, (height, width), self.min_area, self.detect_corrupted)
+                outer_contours = ip.filter_contours_on_frame(outer_contours, (height, width), self.min_area,
+                                                             self.detect_corrupted)
+                outer_contours = ip.remove_small_contours(outer_contours)
 
                 if self.create_json:
-                    if not cv.imwrite(f"{self.output_images_path}/{filename}", img):
-                        print(f"FAILED to save image: {self.output_images_path}/{filename}")
+                    if not cv.imwrite(os.path.join(self.output_images_path, filename), img):
+                        print(f"FAILED to save image: {os.path.join(self.output_images_path, filename)}")
 
                 img_with = img.copy()
-                if not contours:
+                if len(outer_contours) == 0:
                     if self.create_json:
                         self.coco_data = fm.convert_contours_to_coco([], [], height, width,
                                                                      filename,
@@ -99,61 +127,54 @@ class Contours(BaseImageProcessing):
                                                                      self.coco_data)
                     cv.line(img_with, (0, 0), (width - 1, height - 1), (0, 0, 255), 5)
                     cv.line(img_with, (0, height - 1), (width - 1, 0), (0, 0, 255), 5)
-                    cv.line(img_without, (0, 0), (width - 1, height - 1), (0, 0, 255), 5)
-                    cv.line(img_without, (0, height - 1), (width - 1, 0), (0, 0, 255), 5)
+                    if not cv.imwrite(
+                            os.path.join(self.output_segmented_path, "results", filename.replace('.bmp', '.png')),
+                            img_with):
+                        print(
+                            f"FAILED to save image: {os.path.join(self.output_segmented_path, 'results', filename.replace('.bmp', '.png'))}")
+
                 else:
-
-                    inner_contours = []
-                    outer_contours = []
                     img_without = img.copy()
-                    if self.contours_state == "all" or self.contours_state == "select":
-
-                        for i, contour in enumerate(contours):
-                            # Získání indexu rodiče pro aktuální konturu
-                            parent_index = hierarchy[0, i, 3]  # Correctly access the parent index
-                            if parent_index == -1:
-                                # Kontura bez rodiče - vnější kontura
-                                cv.drawContours(img_with, [contour], -1, (0, 0, 255), 2)  # Červeně pro vnější kontury
-                                outer_contours.append(contour)
-                            else:
-                                # Kontrola, zda má rodič této kontury také rodiče (kontury druhého řádu)
-                                grandparent_index = hierarchy[0][parent_index][3]
-
-                                if grandparent_index == -1:
-                                    # Kontura s jedním předkem - vnitřní kontura
-                                    cv.drawContours(img_with, [contour], -1, (255, 0, 0),
-                                                    2)  # Modře pro vnitřní kontury
-                                    inner_contours.append(contour)
-
-                    if len(outer_contours) == 0:
-                        outer_contours = contours
-
-                    mask_with = np.zeros_like(img_gray)
                     mask_without = np.zeros_like(img_gray)
-                    if self.contours_state == "all" or self.contours_state == "select":
-                        # Fill the mask for outer contours
-                        for contour in outer_contours:
-                            cv.fillPoly(mask_with, [contour], 255)
 
-                        # Create and fill a separate mask for inner contours
+                    for contour in outer_contours:
+                        contour = np.array(contour, dtype=np.int32)
+                        cv.fillPoly(mask_without, [contour], 255)
+
+                    if self.contours_state == "all" or self.contours_state == "select":
                         inner_mask = np.zeros_like(img_gray)
                         for contour in inner_contours:
+                            contour = np.array(contour, dtype=np.int32)
                             cv.fillPoly(inner_mask, [contour], 255)
 
                         # Find intersection between outer and inner masks
-                        intersection = mask_with & inner_mask
+                        intersection = mask_without & inner_mask
+                        inner_contours, _ = cv.findContours(intersection, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_SIMPLE)
+                        inner_contours = ip.remove_small_contours(inner_contours)
 
                         # Subtract intersection from the outer contours mask
-                        mask_with = mask_with - intersection
-
-                        # For 'mask_without', only fill the outer contours on a black background
-                    for contour in outer_contours:
-                        cv.fillPoly(mask_without, [contour], 255)
+                        mask_with = mask_without - intersection
 
+                        for contour in inner_contours:
+                            cv.drawContours(img_with, [contour], -1, (255, 0, 0), 2)
 
                     for index, contour in enumerate(outer_contours):
-                        if not self.contours_state == "all":
-                            cv.drawContours(img_without, [contour], -1, [0, 0, 255], 2)
+                        contour_budding = self.evaluate(contour)
+
+                        if not contour_budding == "no_split":
+                            color = (0, 255, 0)
+                        else:
+                            color = (0, 0, 255)
+
+                        if self.contours_state == "all":
+                            cv.drawContours(img_with, [contour], -1, color, 2)
+                        elif self.contours_state == "no":
+                            cv.drawContours(img_without, [contour], -1, color, 2)
+                        else:
+                            cv.drawContours(img_without, [contour], -1, color, 2)
+                            cv.drawContours(img_with, [contour], -1, color, 2)
+
+                        self.save_images_and_masks(filename, (img_without, img_with, mask_without, mask_with), dialog)
 
                         if self.calculate_properties:
                             contour_data = {
@@ -172,29 +193,6 @@ class Contours(BaseImageProcessing):
                                                                      self.counter,
                                                                      self.coco_data)
 
-                if self.contours_state == "select":
-                    dialog.update_selection_dialog(img_without, img_with, mask_without, mask_with,
-                                                   f"{self.output_segmented_path}/results/{filename.replace('.bmp', '.png')}",
-                                                   f"{self.output_segmented_path}/masks/{filename.replace('.bmp', '.png')}", self.counter)
-                elif self.contours_state == "all":
-                    if not cv.imwrite(f"{self.output_segmented_path}/masks/{filename.replace('.bmp', '.png')}",
-                                      mask_with):
-                        print(
-                            f"FAILED to save mask: {self.output_segmented_path}/masks/{filename.replace('.bmp', '.png')}")
-                    if not cv.imwrite(f"{self.output_segmented_path}/results/{filename.replace('.bmp', '.png')}",
-                                      img_with):
-                        print(
-                            f"FAILED to save image: {self.output_segmented_path}/results/{filename.replace('.bmp', '.png')}")
-                else:
-                    if not cv.imwrite(f"{self.output_segmented_path}/masks/{filename.replace('.bmp', '.png')}",
-                                      mask_without):
-                        print(
-                            f"FAILED to save mask: {self.output_segmented_path}/masks/{filename.replace('.bmp', '.png')}")
-                    if not cv.imwrite(f"{self.output_segmented_path}/results/{filename.replace('.bmp', '.png')}",
-                                      img_without):
-                        print(
-                            f"FAILED to save image: {self.output_segmented_path}/results/{filename.replace('.bmp', '.png')}")
-
                 if self.progress_window:
                     progress_text = f"{self.counter}/{total_files}"
                     self.progress_window.update_progress(progress_text)
@@ -216,7 +214,7 @@ class Contours(BaseImageProcessing):
                 'LengthMajorDiameterThroughCentroid', 'LengthMinorDiameterThroughCentroid',
                 'Perimeter', 'Solidity', 'Sphericity'
             ])
-            df.to_excel(f"{self.excel_address}/contour_properties.xlsx")
+            df.to_excel(os.path.join(self.excel_address, "contour_properties.xlsx"))
 
         if self.create_json:
             with open(self.output_json_path, "w") as json_file:
@@ -228,6 +226,34 @@ class Contours(BaseImageProcessing):
         if self.progress_window:
             self.progress_window.update_progress("FINISHED")
 
+    def save_images_and_masks(self, filename, data, dialog=None):
+        img_without, img_with, mask_without, mask_with = data
+        # Construct the base paths for results and masks
+        results_path = os.path.join(self.output_segmented_path, "results")
+        masks_path = os.path.join(self.output_segmented_path, "masks")
+
+        # Replace the file extension from '.bmp' to '.png'
+        new_filename = f"{os.path.splitext(filename)[0]}.png"
+
+        # Full paths for the new image and mask files
+        new_image_path = os.path.join(results_path, new_filename)
+        new_mask_path = os.path.join(masks_path, new_filename)
+
+        if self.contours_state == "select" and dialog:
+            dialog.update_selection_dialog(img_without, img_with, mask_without, mask_with,
+                                           new_image_path, new_mask_path,
+                                           self.counter)
+        elif self.contours_state == "all":
+            if not cv.imwrite(new_mask_path, mask_with):
+                print(f"FAILED to save mask: {new_mask_path}")
+            if not cv.imwrite(new_image_path, img_with):
+                print(f"FAILED to save image: {new_image_path}")
+        else:
+            if not cv.imwrite(new_mask_path, mask_without):
+                print(f"FAILED to save mask: {new_mask_path}")
+            if not cv.imwrite(new_image_path, img_without):
+                print(f"FAILED to save image: {new_image_path}")
+
 
 class IoU(BaseImageProcessing):
     def __init__(self, adresa_output, project, algorithm, contours_state,
@@ -237,14 +263,11 @@ class IoU(BaseImageProcessing):
         self.algorithm = algorithm
         self.contours_state = contours_state
         self.detect_corrupted = detect_corrupted
-        self.adresa_output = f"{adresa_output}/{project}/IoU"
-        self.adresa_plots = f"{adresa_output}/{project}/IoU/plots/{self.algorithm}"
+        self.adresa_output = os.path.join(adresa_output, project, "IoU")
+        self.adresa_plots = os.path.join(self.adresa_output, "plots", self.algorithm)
 
         fm.create_directory(self.adresa_output)
 
-        # for mask, name in zip(self.masks, self.img_names):
-        #    cv.imwrite(f"img_print/{name}",mask)
-
     def process_and_compute_iou(self, ref_mask, img, img_name, parameters, save, lock):
         # Convert the mask tensor and image tensor to numpy arrays
         ref_mask = ref_mask.numpy()
@@ -255,15 +278,14 @@ class IoU(BaseImageProcessing):
 
         # Apply the segmentation algorithm
         img_binary, inner_contours_mask = self.apply_segmentation_algorithm(
-            self.algorithm, parameters, img_gray, self.contours_state,
-            self.detect_corrupted)
+            self.algorithm, parameters, img_gray, self.contours_state)
 
         # Further processing and IoU, TPR, PPV computation
         if self.contours_state == "all" or self.contours_state == "select":
             intersection = inner_contours_mask & img_binary
             img_binary = img_binary - intersection
 
-        contours, hierarchy = cv.findContours(img_binary, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_NONE)
+        contours, _ = cv.findContours(img_binary, cv.RETR_EXTERNAL, cv.CHAIN_APPROX_NONE)
 
         mask = np.zeros_like(img_binary, dtype=np.uint8)
         if not contours:
@@ -352,7 +374,7 @@ class IoU(BaseImageProcessing):
         detect_corrupted_string = "WITH_detecting_corrupted" if self.detect_corrupted else "WITHOUT_detecting_corrupted"
 
         with open(
-                f"{self.adresa_output}/results_{self.project}_{self.algorithm}_IoU_{round(averageIoU * 100, 2)}_{contours_state_string}_{detect_corrupted_string}.json",
+                os.path.join(self.adresa_output, f"results_{self.project}_{self.algorithm}_IoU_{round(averageIoU * 100, 2)}_{contours_state_string}_{detect_corrupted_string}.json"),
                 "w") as json_file:
             json.dump(json_data, json_file, indent=4, cls=NumpyEncoder)
 
