@@ -24,45 +24,22 @@ class SelectionDialog:
         self.dialog.title("Software Selection")
         self.dialog.geometry("600x200")
 
-        self.main_section_label = tk.Label(self.dialog, text="Main software", font=("Helvetica", 12, "bold"))
-        self.main_section_label.pack()
+        tk.Label(self.dialog, text="Main software", font=("Helvetica", 12, "bold")).pack()
+        tk.Button(self.dialog, text="Spheroids Segmentation", command=self.open_segmentation_gui).pack(pady=10)
+        tk.Button(self.dialog, text="Spheroids Quantification", command=self.open_quantification_gui).pack(pady=10)
 
-        seg_button = tk.Button(self.dialog, text="Spheroids Segmentation", command=self.open_segmentation_gui)
-        seg_button.pack(pady=10)
-
-        quant_button = tk.Button(self.dialog, text="Spheroids Quantification", command=self.open_quantification_gui)
-        quant_button.pack(pady=10)
-
-        self.address_separator = tk.Frame(self.dialog, height=2, bd=1, relief=tk.SUNKEN)
-        self.address_separator.pack(fill=tk.X, padx=5, pady=5)
-
-        self.utils_section_label = tk.Label(self.dialog, text="Utils", font=("Helvetica", 12, "bold"))
-        self.utils_section_label.pack()
+        tk.Frame(self.dialog, height=2, bd=1, relief=tk.SUNKEN).pack(fill=tk.X, padx=5, pady=5)
+        tk.Label(self.dialog, text="Utils", font=("Helvetica", 12, "bold")).pack()
 
         button_frame = tk.Frame(self.dialog)
         button_frame.pack(pady=10)
+        tk.Button(button_frame, text="Convert between COCO 1.0 and Masks", command=self.open_conversion_gui).grid(row=0, column=0, padx=(0, 10))
+        tk.Button(button_frame, text="Create dataset folder (merge directories)", command=self.open_image_processing_dialog).grid(row=0, column=1, padx=(10, 0))
 
-        convert_button = tk.Button(button_frame, text="Convert between COCO 1.0 and Masks",
-                                   command=self.open_conversion_gui)
-        image_processing_btn = tk.Button(button_frame, text="Create dataset folder (merge directories)",
-                                         command=self.open_image_processing_dialog)
-
-        # Place buttons in the frame using grid
-        convert_button.grid(row=0, column=0, padx=(0, 10))  # Padding to the right
-        image_processing_btn.grid(row=0, column=1, padx=(10, 0))  # Padding to the left
-
-        # Adjust the frame's column configuration to center the buttons
         button_frame.grid_columnconfigure(0, weight=1)
         button_frame.grid_columnconfigure(1, weight=1)
-
         self.dialog.protocol("WM_DELETE_WINDOW", self.on_close)
 
-    def show(self):
-        self.dialog.deiconify()
-
-    def hide(self):
-        self.dialog.withdraw()
-
     def open_segmentation_gui(self):
         self.hide()
         SpheroidSegmentationGUI(self)
@@ -76,12 +53,33 @@ class SelectionDialog:
         ConversionDialog(self)
 
     def open_image_processing_dialog(self):
-        self.hide()  # Optionally hide the selection dialog
+        self.hide()
         ImageProcessingDialog(self)
 
+    def show(self):
+        self.dialog.deiconify()
+
+    def hide(self):
+        self.dialog.withdraw()
+
     def on_close(self):
         self.root.quit()
 
+def browse_directory(var, title, label):
+    directory_path = filedialog.askdirectory()
+    # Normalize the path to ensure consistency across different OS
+    directory_path = os.path.normpath(directory_path)
+    var.set(directory_path)
+    label.config(text=f"{title}: {shorten_path(directory_path)}")
+
+def shorten_path(path, max_length=40):
+    if len(path) > max_length:
+        return '...' + path[-max_length + 3:]
+    return path
+
+def update_addresses(annotations_address):
+    return os.path.join(annotations_address, "annotations"), os.path.join(annotations_address, "images")
+
 
 class ImageProcessingDialog:
     def __init__(self, selection_dialog):
@@ -348,13 +346,6 @@ class ConversionDialog:
         return path
 
 
-def update_addresses(annotationsAddress):
-    annotation_address = os.path.join(annotationsAddress, "annotations")
-    images_address = os.path.join(annotationsAddress, "images")
-
-    return annotation_address, images_address
-
-
 class ProcessingProgressWindow(tk.Toplevel):
     def __init__(self, master):
         tk.Toplevel.__init__(self, master)
@@ -469,26 +460,15 @@ class ParameterEntryDialog(tk.Toplevel):
         self.parameters = new_parameters
 
 
-def shorten_path(path, max_length=40):
-    if len(path) > max_length:
-        return '...' + path[-max_length + 3:]
-    return path
-
-
 def browse_file(var, title, label):
     file_path = filedialog.askopenfilename()
+    # Normalize the file path
+    file_path = os.path.normpath(file_path)
     var.set(file_path)
     shortened_path = shorten_path(file_path)
     label.config(text=f"{title}: {shortened_path}")
 
 
-def browse_directory(var, title, label):
-    directory_path = filedialog.askdirectory()
-    var.set(directory_path)
-    shortened_path = shorten_path(directory_path)
-    label.config(text=f"{title}: {shortened_path}")
-
-
 class SpheroidQuantificationGUI:
     def __init__(self, selection_dialog):
         self.selection_dialog = selection_dialog
@@ -586,7 +566,7 @@ class SpheroidQuantificationGUI:
         ]
         df = pd.DataFrame(all_contour_data, columns=columns)
         output_path = self.output_path_quantification.get()
-        df.to_excel(f"{output_path}/contour_properties.xlsx")
+        df.to_excel(os.path.join(output_path,"contour_properties.xlsx"))
 
         # Close the progress dialog
         progress_dialog.destroy()
@@ -600,18 +580,39 @@ class SpheroidSegmentationGUI:
     def __init__(self, selection_dialog):
         self.selection_dialog = selection_dialog
         self.dialog = tk.Toplevel(selection_dialog.root)
-        self.dialog.title("Spheroid Quantification")
+        self.dialog.title("Spheroid Segmentation")
 
-        back_button = tk.Button(self.dialog, text="Back", command=self.go_back)
+        self.canvas = tk.Canvas(self.dialog)
+        self.scrollbar = tk.Scrollbar(self.dialog, orient="vertical", command=self.canvas.yview)
+        self.scrollable_frame = ttk.Frame(self.canvas)
+
+        self.scrollable_frame.bind(
+            "<Configure>",
+            lambda e: self.canvas.configure(
+                scrollregion=self.canvas.bbox("all")
+            )
+        )
+
+        # Step 3: Create a window in the canvas for your scrollable_frame
+        self.canvas.create_window((0, 0), window=self.scrollable_frame, anchor="nw")
+
+        # Step 4: Configure the canvas to use the scrollbar
+        self.canvas.configure(yscrollcommand=self.scrollbar.set)
+
+        # Packing the canvas and scrollbar to the dialog
+        self.canvas.pack(side="left", fill="both", expand=True)
+        self.scrollbar.pack(side="right", fill="y")
+
+        back_button = tk.Button(self.scrollable_frame, text="Back", command=self.go_back)
         back_button.pack(anchor='nw', padx=10, pady=5)
         self.loaded_parameters = None
         self.loaded_method = None
 
         # Adresní sekce
-        self.address_section_label = tk.Label(self.dialog, text="Address Settings", font=("Helvetica", 12, "bold"))
+        self.address_section_label = tk.Label(self.scrollable_frame, text="Address Settings", font=("Helvetica", 12, "bold"))
         self.address_section_label.pack()
 
-        self.notebook = ttk.Notebook(self.dialog)
+        self.notebook = ttk.Notebook(self.scrollable_frame)
         self.coco_tab = ttk.Frame(self.notebook)  # První záložka pro COCO
         self.mask_tab = ttk.Frame(self.notebook)  # Druhá záložka pro masky
 
@@ -657,7 +658,7 @@ class SpheroidSegmentationGUI:
         self.image_dataset_path = tk.StringVar()
 
         # File Dialog pro výsledné segmentované obrázky (změna na askdirectory)
-        self.dataset_address_button = tk.Button(self.dialog,
+        self.dataset_address_button = tk.Button(self.scrollable_frame,
                                                 text="Dataset of all images address (folder of images you want to segment)",
                                                 command=lambda: browse_directory(self.dataset_path,
                                                                                  "Selected dataset path",
@@ -667,11 +668,11 @@ class SpheroidSegmentationGUI:
         self.dataset_path = tk.StringVar()
 
         # Indikátor pro vyplnění adresy
-        self.dataset_address_label = tk.Label(self.dialog, text="Dataset Directory: Not selected")
+        self.dataset_address_label = tk.Label(self.scrollable_frame, text="Dataset Directory: Not selected")
         self.dataset_address_label.pack()
 
         # File Dialog pro výsledné segmentované obrázky (změna na askdirectory)
-        self.output_address_button = tk.Button(self.dialog,
+        self.output_address_button = tk.Button(self.scrollable_frame,
                                                text="Output address (folder where to save the output)",
                                                command=lambda: browse_directory(self.output_path,
                                                                                 "Selected output path",
@@ -681,31 +682,31 @@ class SpheroidSegmentationGUI:
         self.output_path = tk.StringVar()
 
         # Indikátor pro vyplnění adresy
-        self.output_address_label = tk.Label(self.dialog, text="Output Directory: Not selected")
+        self.output_address_label = tk.Label(self.scrollable_frame, text="Output Directory: Not selected")
         self.output_address_label.pack()
 
         # Oddělovací čára mezi adresní a metody sekce
-        self.address_separator = tk.Frame(self.dialog, height=2, bd=1, relief=tk.SUNKEN)
+        self.address_separator = tk.Frame(self.scrollable_frame, height=2, bd=1, relief=tk.SUNKEN)
         self.address_separator.pack(fill=tk.X, padx=5, pady=5)
 
         # Metodická sekce
-        self.method_section_label = tk.Label(self.dialog, text="Method Settings", font=("Helvetica", 12, "bold"))
+        self.method_section_label = tk.Label(self.scrollable_frame, text="Method Settings", font=("Helvetica", 12, "bold"))
         self.method_section_label.pack()
 
         # Textbox pro název projektu
-        self.project_name_label = tk.Label(self.dialog, text="Project Name:")
+        self.project_name_label = tk.Label(self.scrollable_frame, text="Project Name:")
         self.project_name_label.pack()
-        self.project_name_entry = tk.Entry(self.dialog)
+        self.project_name_entry = tk.Entry(self.scrollable_frame)
         self.project_name_entry.pack()
 
         # Tlačítko pro načtení parametrů z JSON souboru
-        self.load_parameters_button = tk.Button(self.dialog,
+        self.load_parameters_button = tk.Button(self.scrollable_frame,
                                                 text="I already know the parameters (load JSON file with parameters)",
                                                 command=self.load_and_run_parameters)
         self.load_parameters_button.pack()
 
         # Create a frame to contain the label and button
-        self.parameters_frame = tk.Frame(self.dialog)
+        self.parameters_frame = tk.Frame(self.scrollable_frame)
         self.parameters_frame.pack()
 
         # Initially disable the "Parameters loaded" label and the "Cancel" button
@@ -717,7 +718,7 @@ class SpheroidSegmentationGUI:
         self.cancel_button.pack(side=tk.LEFT)
 
         # Checkboxy pro výběr metod
-        self.methods_frame = tk.Frame(self.dialog)
+        self.methods_frame = tk.Frame(self.scrollable_frame)
         self.methods_frame.pack()
         self.method_labels = ["Sauvola", "Niblack", "Gaussian"]
 
@@ -731,7 +732,7 @@ class SpheroidSegmentationGUI:
             self.methods_checkboxes.append((method_checkbox, method_var))
 
         # Textová pole pro zadání parametrů
-        self.parameters_frame = tk.Frame(self.dialog)
+        self.parameters_frame = tk.Frame(self.scrollable_frame)
         self.parameters_frame.pack()
         self.learning_rate_label = tk.Label(self.parameters_frame, text="Learning Rate:")
         self.learning_rate_label.grid(row=0, column=0, padx=5, pady=5)
@@ -754,16 +755,16 @@ class SpheroidSegmentationGUI:
         self.batch_size_entry.insert(0, "10")
 
         # Oddělovací čára mezi adresní a metody sekce
-        self.method_separator = tk.Frame(self.dialog, height=2, bd=1, relief=tk.SUNKEN)
+        self.method_separator = tk.Frame(self.scrollable_frame, height=2, bd=1, relief=tk.SUNKEN)
         self.method_separator.pack(fill=tk.X, padx=5, pady=5)
 
         # Hole Finding Settings Section
-        self.hole_finding_section_label = tk.Label(self.dialog, text="Hole Finding Settings",
+        self.hole_finding_section_label = tk.Label(self.scrollable_frame, text="Hole Finding Settings",
                                                    font=("Helvetica", 12, "bold"))
         self.hole_finding_section_label.pack()
 
         # Frame for the checkboxes
-        self.hole_finding_frame = tk.Frame(self.dialog)
+        self.hole_finding_frame = tk.Frame(self.scrollable_frame)
         self.hole_finding_frame.pack()
 
         # Variables for the checkboxes
@@ -800,13 +801,13 @@ class SpheroidSegmentationGUI:
         self.view_select_checkbox.pack(anchor='w')
 
         # Oddělovací čára mezi adresní a metody sekce
-        self.settings_separator = tk.Frame(self.dialog, height=2, bd=1, relief=tk.SUNKEN)
+        self.settings_separator = tk.Frame(self.scrollable_frame, height=2, bd=1, relief=tk.SUNKEN)
         self.settings_separator.pack(fill=tk.X, padx=5, pady=5)
 
-        self.other_section_label = tk.Label(self.dialog, text="Other Settings", font=("Helvetica", 12, "bold"))
+        self.other_section_label = tk.Label(self.scrollable_frame, text="Other Settings", font=("Helvetica", 12, "bold"))
         self.other_section_label.pack()
 
-        checkbox_frame = tk.Frame(self.dialog)
+        checkbox_frame = tk.Frame(self.scrollable_frame)
         checkbox_frame.pack()
 
         self.detect_corrupted_var = tk.BooleanVar()
@@ -830,12 +831,27 @@ class SpheroidSegmentationGUI:
         self.checkbox_calculate_contours.pack(side=tk.LEFT)
 
         # Tlačítko pro spuštění
-        self.run_button = tk.Button(self.dialog, text="Run", command=self.run_method)
+        self.run_button = tk.Button(self.scrollable_frame, text="Run", command=self.run_method)
         self.run_button.pack()
 
+        self.dialog.update_idletasks()  # Update the window to calculate sizes
+
+        # Calculate and set the window size
+        self.set_initial_window_size()
+
         self.dialog.protocol("WM_DELETE_WINDOW", self.on_close)
         self.stop_event = threading.Event()
 
+    def set_initial_window_size(self):
+        # Calculate required width for the content
+        required_width = self.scrollable_frame.winfo_reqwidth() + self.scrollbar.winfo_width()
+
+        # Get the maximum available height on the screen, leaving some space
+        max_height = self.dialog.winfo_screenheight() - 100  # 100 pixels less to ensure it fits
+
+        # Set the initial size of the window to fit the content width and limit its height
+        self.dialog.geometry(f"{required_width}x{max_height}")
+
     def go_back(self):
         self.dialog.destroy()
         self.selection_dialog.show()
@@ -1004,13 +1020,17 @@ class SpheroidSegmentationGUI:
                 annotation_address = self.coco_annotation_path.get()
                 annotation_address = fm.unzip(annotation_address)
                 annotations_address, images_address = update_addresses(annotation_address)
-                annotation_data = fm.load_annotations(
-                    os.path.join(annotations_address, 'instances_default.json'), images_address)
+                annotations_address = os.path.join(annotations_address, 'instances_default.json')
+                print(f"Loading annotations JSON from: {annotations_address}")
+                print(f"Loading reference Images from: {images_address}")
+                annotation_data = fm.load_annotations(annotations_address, images_address)
                 print(f"Loaded {len(annotation_data)} annotated images")
             elif current_tab == self.mask_tab:
-                mask_adress = self.masks_annotation_path.get()
+                mask_address = self.masks_annotation_path.get()
                 images_address = self.image_dataset_path.get()
-                annotation_data = fm.load_masks_from_images(mask_adress, images_address)
+                print(f"Loading reference Masks from: {mask_address}")
+                print(f"Loading reference Images from: {images_address}")
+                annotation_data = fm.load_masks_from_images(mask_address, images_address)
                 print(f"Loaded {len(annotation_data)} annotated images")
         else:
             annotation_data = None
