@@ -17,6 +17,8 @@ class MyDataset(torch.utils.data.Dataset):
 
     def __getitem__(self, idx):
         mask, image, name = self.data[idx]
+        if mask is None or image is None or name is None:
+            raise ValueError(f"Data at index {idx} contains None. Please check the input dataset.")
         return mask, image, name
 
 
@@ -47,8 +49,8 @@ class GradientDescent:
             self.cont_parameters.update({"k": 0.2})
             self.cont_param_ranges.update({"k": [0.0, 0.35]})
 
-        self.cont_parameters.update({"min_area": 5000, "sigma": 1.0, "std_k": 0.5})
-        self.cont_param_ranges.update({"min_area": [0, 50000], "sigma": [0.0, 5.0], "std_k": [0.0, 3.0]})
+        self.cont_parameters.update({"min_area": 0.005, "sigma": 1.0, "std_k": 0.5})
+        self.cont_param_ranges.update({"min_area": [0.0, 0.1], "sigma": [0.0, 5.0], "std_k": [0.0, 3.0]})
 
         self.disc_parameters = {"dilation_size": 2}
         self.disc_param_ranges = {"dilation_size": [0, 10]}
